use strict;
use warnings FATAL => 'all';
use Test::More tests => 1;

use_ok('Dependencies::Searcher', "Use Dependencies::Searcher");
# use_ok('App::Ack', "Use Ack");

diag( "Testing Dependencies::Searcher $Dependencies::Searcher::VERSION, Perl $], $^X" );


